@@ -84,6 +84,7 @@ struct _YelpDocumentPriv {
     Hash   *page_ids;      /* Mapping of fragment IDs to real page IDs */
     Hash   *titles;        /* Mapping of page IDs to titles */
     Hash   *descs;         /* Mapping of page IDs to descs */
+    Hash   *keywords;      /* Mapping of page IDs to keywords */
     Hash   *icons;         /* Mapping of page IDs to icons */
     Hash   *mime_types;    /* Mapping of page IDs to mime types */
     Hash   *contents;      /* Mapping of page IDs to string content */
@@ -313,6 +314,7 @@ yelp_document_init (YelpDocument *document)
     priv->page_ids = hash_new (g_free );
     priv->titles = hash_new (g_free);
     priv->descs = hash_new (g_free);
+    priv->keywords = hash_new (g_free);
     priv->icons = hash_new (g_free);
     priv->mime_types = hash_new (g_free);
     priv->contents = hash_new ((GDestroyNotify) str_unref);
@@ -361,6 +363,7 @@ yelp_document_finalize (GObject *object)
     hash_free (document->priv->page_ids);
     hash_free (document->priv->titles);
     hash_free (document->priv->descs);
+    hash_free (document->priv->keywords);
     hash_free (document->priv->icons);
     hash_free (document->priv->mime_types);
 
@@ -750,6 +753,41 @@ yelp_document_set_page_desc (YelpDocument *document,
     g_mutex_unlock (&document->priv->mutex);
 }
 
+gchar *
+yelp_document_get_page_keywords (YelpDocument *document,
+                                 const gchar  *page_id)
+{
+    gchar *real, *ret = NULL;
+
+    g_assert (document != NULL && YELP_IS_DOCUMENT (document));
+
+    if (page_id != NULL && g_str_has_prefix (page_id, "search="))
+        return NULL;
+
+    g_mutex_lock (&document->priv->mutex);
+    real = hash_lookup (document->priv->page_ids, page_id);
+    if (real) {
+	ret = hash_lookup (document->priv->keywords, real);
+	if (ret)
+	    ret = g_strdup (ret);
+    }
+    g_mutex_unlock (&document->priv->mutex);
+
+    return ret;
+}
+
+void
+yelp_document_set_page_keywords (YelpDocument *document,
+                                 const gchar  *page_id,
+                                 const gchar  *keywords)
+{
+    g_assert (document != NULL && YELP_IS_DOCUMENT (document));
+
+    g_mutex_lock (&document->priv->mutex);
+    hash_replace (document->priv->keywords, page_id, g_strdup (keywords));
+    g_mutex_unlock (&document->priv->mutex);
+}
+
 gchar *
 yelp_document_get_page_icon (YelpDocument *document,
                              const gchar  *page_id)
