@@ -307,7 +307,7 @@ window_construct (YelpWindow *window)
         { "yelp-window-ctrll",  action_ctrll,        NULL, NULL, NULL },
     };
 
-    gtk_window_set_icon_name (GTK_WINDOW (window), "help-browser");
+    gtk_window_set_icon_name (GTK_WINDOW (window), "org.gnome.Yelp");
 
     g_object_get (gtk_settings_get_default (),
                   "gtk-dialogs-use-header", &priv->use_header,
@@ -363,6 +363,7 @@ window_construct (YelpWindow *window)
 
     menu = g_menu_new ();
     section = g_menu_new ();
+    g_menu_append (section, _("New Window"), "win.yelp-window-new");
     g_menu_append (section, _("Find…"), "win.yelp-window-find");
     g_menu_append (section, _("Print…"), "win.yelp-view-print");
     g_menu_append_section (menu, NULL, G_MENU_MODEL (section));
@@ -374,6 +375,12 @@ window_construct (YelpWindow *window)
     g_menu_append_section (menu, NULL, G_MENU_MODEL (section));
     g_object_unref (section);
 
+    section = g_menu_new ();
+    g_menu_append (section, _("Larger Text"), "app.yelp-application-larger-text");
+    g_menu_append (section, _("Smaller Text"), "app.yelp-application-smaller-text");
+    g_menu_append_section (menu, NULL, G_MENU_MODEL (section));
+    g_object_unref (section);
+
     section = g_menu_new ();
     g_menu_append (section, _("All Help"), "win.yelp-window-go-all");
     g_menu_append_section (menu, NULL, G_MENU_MODEL (section));
